#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

from pyramid.view import view_config
from z3c.form import button, field
from zope.interface import Interface

from pyams_content_es import _
# import interfaces
from pyams_content_es.interfaces import IContentIndexerUtility
# import packages
from pyams_form.form import AJAXAddForm, AJAXEditForm, ajax_config
from pyams_form.interfaces.form import IWidgetsSuffixViewletsManager
from pyams_form.schema import CloseButton
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.interfaces.viewlet import ITableItemColumnActionsMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_template.template import template_config
from pyams_utils.interfaces import MANAGE_SYSTEM_PERMISSION, VIEW_SYSTEM_PERMISSION
from pyams_viewlet.viewlet import Viewlet, viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyams_zmi.layer import IAdminLayer
from pyams_zmi.zmi.control_panel import UtilitiesTable


@pagelet_config(name='properties.html', context=IContentIndexerUtility, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class ContentIndexerUtilityPropertiesEditForm(AdminDialogEditForm):
    """Content indexer utility properties edit form"""

    prefix = 'indexer_properties.'

    @property
    def title(self):
        return self.context.__name__
    
    legend = _("Update content indexer properties")
    
    fields = field.Fields(IContentIndexerUtility).select('zodb_name')

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION
    
    
@view_config(name='properties.json', context=IContentIndexerUtility, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class ContentIndexerUtilityPropertiesAJAXEditForm(AJAXEditForm, ContentIndexerUtilityPropertiesEditForm):
    """Content index utility properties edit form, JSON renderer"""


@viewlet_config(name='test-conversion-process.menu', context=IContentIndexerUtility, layer=IAdminLayer,
                view=UtilitiesTable, manager=ITableItemColumnActionsMenu, permission=MANAGE_SYSTEM_PERMISSION)
class ContentIndexerProcessTestMenu(ToolbarMenuItem):
    """Content indexer process test menu"""

    label = _("Test process connection...")
    label_css_class = 'fa fa-fw fa-server'
    url = 'test-indexer-process.html'
    modal_target = True
    stop_propagation = True


class IContentIndexerProcessTestButtons(Interface):
    """Content indexer process test buttons"""

    close = CloseButton(name='close', title=_("Close"))
    test = button.Button(name='test', title=_("Test connection"))


@pagelet_config(name='test-indexer-process.html', context=IContentIndexerUtility, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
@ajax_config(name='text-indexer-process.json', context=IContentIndexerUtility, layer=IPyAMSLayer,
             base=AJAXAddForm)
class ContentIndexerProcessTestForm(AdminDialogAddForm):
    """Content indexer process test form"""

    @property
    def title(self):
        return self.context.__name__

    legend = _("Test content indexer process connection")
    icon_css_class = 'fa fa-fw fa-server'

    prefix = 'test_form.'
    fields = field.Fields(Interface)
    buttons = button.Buttons(IContentIndexerProcessTestButtons)

    edit_permission = MANAGE_SYSTEM_PERMISSION

    @property
    def form_target(self):
        return '#{0}_test_result'.format(self.id)

    def updateActions(self):
        super(ContentIndexerProcessTestForm, self).updateActions()
        if 'test' in self.actions:
            self.actions['test'].addClass('btn-primary')

    def createAndAdd(self, data):
        return self.context.test_process()

    def get_ajax_output(self, changes):
        status, message = changes
        if status == 200:
            return {
                'status': 'success',
                'content': {'html': message},
                'close_form': False
            }
        else:
            return {
                'status': 'info',
                'content': {'html': message},
                'close_form': False
            }


@viewlet_config(name='test-indexer-process.suffix', layer=IAdminLayer, manager=IWidgetsSuffixViewletsManager,
                view=ContentIndexerProcessTestForm, weight=50)
@template_config(template='templates/process-test.pt')
class ContentIndexerProcessTestSuffix(Viewlet):
    """Content indexer process test form suffix"""
